package com.bizmatics.mhfire.controller.web;

import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.mhfire.model.Alert;
import com.bizmatics.mhfire.service.AlertService;
import com.bizmatics.mhfire.service.vo.AlertStatisticsVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 警情模块
 * @author yq
 * @date 2021/5/31 10:05
 */
@RestController
@RequestMapping("/alert")
public class AlertControllerWeb {

    @Autowired
    private AlertService alertService;


    /**
     * 查询当天24小时警情
     *
     * @return
     */
    @GetMapping("/alertStatisticsByHouse")
    public ApiResult<Map<String, List<AlertStatisticsVO>>> getAlertStatisticsByHouse() {
        return ApiResult.success(alertService.getAlertStatisticsByHouse());
    }


    /**
     * 查询每年12个月警情
     *
     * @return
     */
    @GetMapping("/alertStatisticsByMonth")
    public ApiResult<Map<String, List<AlertStatisticsVO>>> getAlertStatisticsByMonth() {
        return ApiResult.success(alertService.getAlertStatisticsByMonth());
    }


    /**
     * 警情详细信息
     *
     * @param current   页数
     * @param size      条数
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return
     */
    @GetMapping("/page")
    public ApiResult<CommonPage<Alert>> page(@RequestParam Integer current,
                                             @RequestParam Integer size,
                                             @RequestParam(required = false) Date startTime,
                                             @RequestParam(required = false) Date endTime) {
        return ApiResult.success(alertService.page(current, size, startTime, endTime));
    }

    /**
     * 查询不同警情(社会救助，警情，火灾)的占比
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return
     */
    @GetMapping("/alertStatistics")
    public ApiResult<List<AlertStatisticsVO>> getAlertStatistics(@RequestParam(required = false) Date startTime,
                                                                 @RequestParam(required = false) Date endTime) {
        return ApiResult.success(alertService.getAlertStatistics(startTime, endTime));
    }

    @PostMapping("/add")
    public void add(@RequestBody List<Alert> alert){
        alertService.saveBatch(alert);
    }
}
