package com.bizmatics.mhfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.util.DateUtils;
import com.bizmatics.common.core.util.StringUtils;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.mhfire.model.Alert;
import com.bizmatics.mhfire.persistence.mapper.AlertMapper;
import com.bizmatics.mhfire.service.AlertService;
import com.bizmatics.mhfire.service.util.Arith;
import com.bizmatics.mhfire.service.vo.AlertStatisticsVO;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author yq
 * @date 2021/5/28 14:53
 */
@Service
public class AlertServiceImpl extends AbstractCrudService<AlertMapper, Alert> implements AlertService {

    private static final String[] ALERT_TYPE = {"火灾","社会救助","抢险救援"};

    @Override
    public Map<String, List<AlertStatisticsVO>> getAlertStatisticsByHouse() {
        Date date = new Date();
        Date startTime = DateUtils.getDayStartTime(date);
        Map<String, List<AlertStatisticsVO>> typeMap = enhanceList(baseMapper.getCountByHorse(startTime, date));
        perfect(typeMap,24);
        return typeMap;
    }

    public void perfect(Map<String, List<AlertStatisticsVO>> typeMap,Integer times){
        for (String type:ALERT_TYPE) {
            if (!typeMap.containsKey(type)){
                typeMap.put(type,new ArrayList<>());
            }
            perfectDate(typeMap.get(type), times);
        }
    }

    public void perfectDate(List<AlertStatisticsVO> list,Integer times){
        for (int i = 1; i <= times; i++) {
            int finalI = i;
            if (list.stream().noneMatch(asv->Integer.parseInt(asv.getMonth()) == finalI)){
                AlertStatisticsVO asv = new AlertStatisticsVO();
                asv.setMonth(Integer.toString(i));
                asv.setNumber(0);
                list.add(asv);
            }
        }
        list.sort(Comparator.comparingInt(x -> Integer.parseInt(x.getMonth())));
    }



    public Map<String, List<AlertStatisticsVO>> enhanceList(List<Map<String,Object>> mapList){
        List<AlertStatisticsVO> list = new ArrayList<>();
        for (Map<String, Object> map:mapList) {
            AlertStatisticsVO alertStatisticsVo = new AlertStatisticsVO();
            alertStatisticsVo.setType(map.get("aType").toString());
            alertStatisticsVo.setMonth(map.get("monthTime").toString());
            alertStatisticsVo.setNumber(Integer.parseInt(map.get("aCount").toString()));
            alertStatisticsVo.setType(map.get("aType").toString());
            list.add(alertStatisticsVo);
        }
        return list.stream().collect(Collectors.groupingBy(AlertStatisticsVO::getType));
    }


    @Override
    public Map<String, List<AlertStatisticsVO>> getAlertStatisticsByMonth() {
        Date date = new Date();
        Date startTime = DateUtils.getDayStartTime(date);
        Map<String, List<AlertStatisticsVO>> typeMap = enhanceList(baseMapper.getCountByMonth(startTime, date));
        perfect(typeMap,12);
        return typeMap;
    }

    @Override
    public CommonPage<Alert> page(Integer current, Integer size, Date startTime, Date endTime) {
        IPage<Alert> page = new Page<>(current, size);
        LambdaQueryWrapper<Alert> queryWrapper = Wrappers.lambdaQuery();
        if (null != startTime && null != endTime){
            queryWrapper.between(Alert::getDcsj,startTime,endTime);
        }
        page = this.page(page, queryWrapper);
        return  this.ToCommonPage(page);
    }

    @Override
    public List<AlertStatisticsVO> getAlertStatistics(Date startTime, Date endTime) {
        List<AlertStatisticsVO> list = new ArrayList<>();
        list.add(getAsV(startTime, endTime, ALERT_TYPE[0]));
        list.add(getAsV(startTime, endTime, ALERT_TYPE[1]));
        list.add(getAsV(startTime, endTime, ALERT_TYPE[2]));
        int sum = list.stream().mapToInt(AlertStatisticsVO::getNumber).sum();
        list.forEach(alertStatisticsVO -> {
            if (0 == sum){
                alertStatisticsVO.setRadio(0.00);
            }else {
                alertStatisticsVO.setRadio(Arith.div(alertStatisticsVO.getNumber(), sum));
            }
        });
        return list;
    }
    public AlertStatisticsVO getAsV(Date startTime,Date endTime,String type){
        AlertStatisticsVO alertStatisticsVo = new AlertStatisticsVO();
        LambdaQueryWrapper<Alert> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(StringUtils.isNotBlank(type),Alert::getAjlx,type);
        if (null != startTime && null != endTime){
            queryWrapper.between(Alert::getDcsj,startTime,endTime);
        }
        alertStatisticsVo.setType(type);
        alertStatisticsVo.setNumber(this.count(queryWrapper));
        return alertStatisticsVo;
    }

}
